'use strict' // requerim strict per a poder emprar les derreres funcionalitats de ecma script


var mongoose = require('mongoose-pagination');// carreguem mongoose pagination
var User = require('../models/user');// carreguem el model de seguiment
var Follow = require('../models/follow'); // carreguem el model de seguiment
var path = require('path');// importem la llibreria per a treballar amb la ruta dels fitxers


// creem controlador de seguiment d'usuaris
function saveFollow(req,res){
	var params = req.body; // recollim els parametres arribats per POST i els guardem
	var follow = new Follow(); // feim la instància d'un nou seguiment
	// en el métode d'autenticació s'han adjuntat les dades d'usuari connectat, per això obtenim id d'usuari que segueix de la seguent manera
	follow.user = req.user.sub; // guardem les dades de l'usuari que segueix
	follow.followed = params.followed; // guardem l'usuari seguit per Post

	follow.save((err, followStored)=>{ // guardem l'usuari amb una funció de callback per a controlar errors
		if(err) return res.status(500).send({message: 'Hi ha hagut un error de seguiment'});
		if(follow.user == follow.followed) return res.status(500).send({message: 'Un usuari no es pot seguir a ell mateix'});
		if(!followStored) return res.status(500).send({message: 'Hi ha hagut un error de seguiment!!'});
		return res.status(200).send({follow: followStored});
	});
}


//funció per a deixar de seguir un usuari
function deleteFollow(req,res){
	var userId = req.user.sub;//obtenim id de l'usuari connectat
	var followId = req.params.id;//usuari que volem deixar de seguir
	Follow.find({'user': userId, 'followed': followId}).remove(err =>{ // esborram el document de seguiment seleccionat
			//si hi ha error a l'hora d'esborrar un seguiment, doncs..
			if(err) return res.status(500).send({message: 'Hi ha hagut un error (El document no ha estat esborrat)'});
			return res.status(200).send({message: 'Document esborrat correctament'});

	})
}

// funció per a obtenir usuaris seguits amb oaginació (itemsPerPage)
function getFollowingUsers(req,res){
	var userId = req.user.sub; //obtenim l'id de l'usuari connectat
	//si arriba un usuari per parámetres, doncs aquest será l'usuari a llistar
	if(req.params.id && req.params.page){
		userId = req.params.id;
	}
	var page = 1; // cream la variable page
	if(req.params.page){ // si ens arriba una pàgina per paràmetres, aquesta serà la pàgina a mostrar
		page = req.params.page;
	}
	//items a mostrar a cada pàgina
	var itemsPerPage = 5;
	//obtenim l'objecte del document d'usuari seguit
	Follow.find({user: userId}).sort('name').populate({path: 'followed'}).paginate(page,itemsPerPage,(err,follows,total)=>{
			if(err) return res.status(500).send({message: 'Hi ha hagut un error'});
			if(!Follow) return res.status(404).send({message: 'No estas seguint a cap usuari'});
			return res.status(200).send({
				total: total,
				pages:  Math.ceil(total/itemsPerPage),
				follows

			});

	});
}

// funció que retorna els usuaris seguits sense paginació

function getFollows(req,res){


var userId = req.user.sub; //obtenim l'ID d'usuari connectat

var find = Follow.find({user: userId}).sort('name'); //obtenim l'objecte del document d'usuari seguit
  
  if(req.params.followed){//si arriba un usuari per parámetres, doncs aquest será l'usuari a llistar
  	find = Follow.find({followed: userId}).sort('name'); // cerquem els usuaris seguits a la base de dades, ordenats pe rnom
  }
	find.populate('user followed').exec((err,follows)=>{ // obtenim l'objecte d'usuari

			if(err) return res.status(500).send({message: 'Hi ha hagut un error'});
			if(!follows) return res.status(404).send({message: 'No hi ha cap usuari que compleixi els requisits'});
			return res.status(200).send({follows});
	});
}


 
// funció per a obtenir usuaris que em segueixen amb paginació (itemsPerPage)
function getFollowedUsers(req,res){
	var userId = req.user.sub; //obtenim l'id de l'usuari connectat
	//si arriba un usuari per parámetres, doncs aquest será l'usuari a llistar
	if(req.params.id && req.params.page){
		userId = req.params.id;
	}
	var page = 1; // cream la variable page
	if(req.params.page){ // si ens arriba una pàgina per paràmetres, aquesta serà la pàgina a mostrar
		page = req.params.page;
	}
	//items a mostrar a cada pàgina
	var itemsPerPage = 5;
	//obtenim l'objecte del document d'usuari seguit
	Follow.find({followed: userId}).sort('name').populate('user followed').paginate(page,itemsPerPage,(err,follows,total)=>{
			if(err) return res.status(500).send({message: 'Hi ha hagut un error'});
			if(!Follow) return res.status(404).send({message: 'No tens cap seguidor'});
			return res.status(200).send({
				total: total,
				pages:  Math.ceil(total/itemsPerPage),
				follows

			});

	});
}




module.exports = {
	saveFollow,
	deleteFollow,
	getFollowingUsers,
	getFollowedUsers,
	getFollows

}