'use strict' // requerim strict per a poder emprar les derreres funcionalitats de ecma script

var moment = require('moment');//carreguem la llibreria moment
var mongoose = require('mongoose-pagination');// carreguem mongoose pagination

var User = require('../models/user');// carreguem el model de seguiment
var Follow = require('../models/follow'); // carreguem el model de seguiment
var Message = require('../models/message'); // carreguem el model de missatge


function saveMessage(req,res){ // funció per a guardar un missatge
	var params = req.body; // recollim les dades arribades per post
	if(!params.text || !params.receiver) return res.status(200).send({message: 'Hi ha camps no omplerts correctament'});
	//cream un nou missatge
	var message = new Message();
	message.emmitter = req.user.sub; // l'emisor es l'usuari connectat
	message.receiver = params.receiver; //el receptor del missatge
	message.text = params.text; //text rebut per paràmetres
	message.created_at = moment().unix(); // moment actual
	message.viewed = 'false';


	//funció de callback per a guardar el missatge
	message.save((err,messageStored) => {

		if(err) return res.status(500).send({message: 'Ups!! hi ha hagut un error'});
		if(!messageStored) return res.status(500).send({message: 'Ups!! Missatge no guardat'});
		return res.status(200).send({message: messageStored});

	})

}

//llistat paginat de missatges rebuts
function getReceivedMessages(req,res){

	var userId = req.user.sub;
	var page = 1;
	if(req.params.page){
		page = req.params.page
	}

	var itemsPerPage = 5; 

	Message.find({receiver: userId}).populate('emmitter', 'name surname image nick _id').sort('-created_at').paginate(page,itemsPerPage,(err,messages,total) =>{
			if(err) return res.status(500).send({message: 'Ups!! hi ha hagut un error'});
			if(!messages) return res.status(500).send({message: 'Ups!!No hi ha missatges'});
			return res.status(200).send({
				total: total,
				pages: Math.ceil(total/itemsPerPage),
				messages
			})

	});
}


//llistat paginat de missatges enviats
function getEmmitMessages(req,res){

	var userId = req.user.sub;
	var page = 1;
	if(req.params.page){
		page = req.params.page
	}

	var itemsPerPage = 5; 

	Message.find({emmitter: userId}).populate('receiver emmitter', 'name surname image nick _id').sort('-created_at').paginate(page,itemsPerPage,(err,messages,total) =>{
			if(err) return res.status(500).send({message: 'Ups!! hi ha hagut un error'});
			if(!messages) return res.status(404).send({message: 'Ups!!No hi ha missatges'});
			return res.status(200).send({
				total: total,
				pages: Math.ceil(total/itemsPerPage),
				messages
			})

	});
}

//funcio per a llistar els missatges no llegits
function getUnreadMessages(req,res){

	var userId = req.user.sub; // obtenim les dades de l'usuari connectat
	var page = 1;
	if(req.params.page){
		page = req.params.page
	}

	var itemsPerPage = 5; 

	Message.countDocuments({receiver: userId, viewed: 'false'}).exec((err,count) =>{
			if(err) return res.status(500).send({message: 'Ups!! hi ha hagut un error'});
			return res.status(200).send({
				total_unread: count
			})

	});



}

//marcar missatges com a llegits
function setReadMessage(req,res){
	var userId = req.user.sub; // obtenim les dades de l'usuari connectat
	//actualitzem el flag del missatge
	Message.update({receiver:userId, viewed: 'false'},{viewed:'true'}, {'multi': true}, (err, messagesUpdated) => {

		if(err) return res.status(500).send({message: 'Ups!! hi ha hagut un error'});
		return res.status(200).send({message: messagesUpdated});


	}) 

}


module.exports = {
	saveMessage,
	getReceivedMessages,
	getEmmitMessages,
	getUnreadMessages,
	setReadMessage

}