'use strict' // requerim strict per a poder emprar les derreres funcionalitats de ecma script

var path = require('path'); //carreguem la llibreria path
var fs = require('fs');//carreguem la llibreria fs
var moment = require('moment');//carreguem la llibreria moment
var mongoose = require('mongoose-pagination');// carreguem mongoose pagination
var Publication = require('../models/publication'); // carreguem el model de publicació
var User = require('../models/user');// carreguem el model de seguiment
var Follow = require('../models/follow'); // carreguem el model de seguiment



//funció per a guardar una nova publicació
function savePublication(req,res){
	var params = req.body; //guardem les dades de l'usuari connectat
	if(!params.text) return res.status(200).send({message: 'No hi ha text a la publicació'})// si no hi ha text, doncs retornam un missatge
	var publication = new Publication(); // guardem l'objecte de la nova publicació
	publication.text = params.text;
	publication.file = 'null'; // per defecte no es necessari guardar cap arxiu o imàtge
	publication.user = req.user.sub; // guardem l'usuari propietari de la publicació
	publication.created_at = moment().unix();// guardem el moment de la creació de la publicació

	publication.save((err,publicationStored) => { // funció de callback per a guardar la publicació

			if(err) return res.status(500).send({message: 'Hi ha hagut un error'});
			if(!publicationStored) res.status(404).send({message:'La publicació no ha estat guardada. Torna a intentar-ho'});
			return res.status(200).send({publication: publicationStored}); // guardem la publicació

	})

}

//mètode per a llistar les publicacions dels usuaris als quals l'usuari identificat segueix(follow)
function getPublications(req,res){
	var page = 1;
	if(req.params.page){
		page = req.params.page // si s'indica llistam la pagina escollida
	}
	var itemsPerPage = 4; // Nombre de documents que tendrà la pàgina
	//cerquem tots els usuaris als quals segueix l'usuari identificat
	Follow.find({user: req.user.sub}).populate('followed').exec((err,follows) => {
			if(err) return res.status(500).send({message: 'Hi ha hagut un error per a obtenir el seguiment'});
			
			// creem un array d'ids d'usuaris
			var follows_clean = [];
			follows.forEach((follow) => {
				follows_clean.push(follow.followed);
			})
			follows_clean.push(req.user.sub);

			// cerquem totes les publicacións dels usuaris pertanyents a l'array follows_clean
			Publication.find({user: {"$in": follows_clean}}).sort('-created_at').populate('user').paginate(page,itemsPerPage,(err,publications,total) =>{
					if(err) return res.status(500).send({message: 'Hi ha hagut un error per a obtenir les publicacions'});
					if(!publications) return res.status(404).send({message: 'Aquest usuari no té publicacions visibles'});
					if(total == 0) return res.status(200).send({message: 'Aquest usuari encara no té publicacions'});
					return res.status(200).send({
						total_items: total,
						pages: Math.ceil(total/itemsPerPage),
						page: page,
						items_per_page:itemsPerPage ,
						publications

					});

			});
			
	});
}

//publcacions de l'usuari seguit
function getPublicationsUser(req,res){
	var page = 1;
	if(req.params.page){
		page = req.params.page // si s'indica llistam la pagina escollida
	}
	var user = req.user.sub;
	if(req.params.user){
		user = req.params.user;
	}
	var itemsPerPage = 4; // Nombre de documents que tendrà la pàgina
	//cerquem tots els usuaris als quals segueix l'usuari identificat
	

			// cerquem totes les publicacións dels usuaris pertanyents a l'array follows_clean
			Publication.find({user: user}).sort('-created_at').populate('user').paginate(page,itemsPerPage,(err,publications,total) =>{
					if(err) return res.status(500).send({message: 'Hi ha hagut un error per a obtenir les publicacions'});
					if(!publications) return res.status(404).send({message: 'Aquest usuari no té publicacions visibles'});
					if(total == 0) return res.status(200).send({message: 'Aquest usuari encara no té publicacions'});
					return res.status(200).send({
						total_items: total,
						pages: Math.ceil(total/itemsPerPage),
						page: page,
						items_per_page:itemsPerPage ,
						publications

					});

			});
			

}



//obtenim una publicació per id
function getPublication(req,res){
	var publicationID = req.params.id; //obtenim l'id passam per paràmetres
		//cerquem la publicació
		Publication.findById(publicationID, (err,publication)=> {
			if(err) return res.status(500).send({message: 'Hi ha hagut un error per a obtenir la publicació'});
			if(!publication) return res.status(404).send({message: 'Aquesta publicació no es pot llistar'});
			return res.status(200).send({publication});
		});

}


//obtenim una publicació per id
function deletePublication(req,res){
	var publicationId = req.params.id; //obtenim l'id passam per paràmetres
		//cerquem la publicació
		// aqui es portarà a terme el control per a poder eliminar publicacions quan un usuari es administrador
		/*if(){

			Publication.findByIdAndRemove(publicationID, (err,publicationRemove)=> {
			if(err) return res.status(500).send({message: 'Hi ha hagut un error per a esborrar la publicació'});
			if(!publicationRemove) return res.status(404).send({message: 'Aquest publicació no ha estat esborrada'});
			return res.status(200).send({publicationRemove});
			});

		}else{ */

			Publication.find({'user':req.user.sub, '_id':publicationId}).deleteOne((err,publicationRemove)=> {
			if(err) return res.status(500).send({message: 'Hi ha hagut un error per a esborrar la publicació'});
			//if(!publicationRemove) return res.status(404).send({message: 'Aquest publicació no ha estat esborrada'});
			return res.status(200).send({publicationRemove});
			});

			
		//}

}


// cream funció per a la pujada d'imàtges de publicacio
function uploadImage(req,res){

	var publicationId = req.params.id;


	if(req.files){

		var file_path = req.files.image.path; // obtenim la ruta de l'arxiu
		var file_split =file_path.split('\\');// separem el path en directoris i subdirectoris
		var file_name = file_split[2]; //obtenim el nom de l'arxiu
		var ext_split = file_name.split('\.');//separem per a obtenir l'extensio
		var file_ext = ext_split[1];//obtenim el tipus d'extensio
		

		//gestionem el control d'extensions permeses
		if(file_ext == 'png' || file_ext == 'jpg' || file_ext == 'jpeg'){


			Publication.findOne({'user': req.user.sub, '_id': publicationId}).exec((err,publication)=>{

				if(publication){

						//actualitzam la publicació
						Publication.findByIdAndUpdate(publicationId, {file: file_name}, {new:true} , (err,publicationUpdated) =>{
						// si la petició no retorna l'objecte actualizat, doncs..
						if(err) return res.status(500).send({message: 'Hi ha agut un error, torna a intentar-ho!!'});
						// Si no s'ha rebut l'actualització de l'usuari..
						if(!publicationUpdated) return res.status(404).send({message: 'Hi ha agut un error, torna a intentar-ho!!'});
						// si tot ha anat be, doncs retorna l'objecte actualitzat
						return res.status(200).send({publication: publicationUpdated});
						})

				}else{
					// no permetem extensions no indicades al condicional
					 return notUploadPublicationImage(res,file_path, 'No disposes de permis per a modificar aquesta publicació!!');
				}
			});

				

		}else{
			// no permetem extensions no indicades al condicional
			 return notUploadPublicationImage(res,file_path, 'Extensió no permesa!!');

		}

	}else{
		return res.status(200).send({message: 'Hi ha hagut un problema!!'});
	}

}


// creem una funció per a no permetre la pujada d'arxius

function notUploadPublicationImage(res,file_path,message){
	fs.unlink(file_path, (err)=>{
		return res.status(200).send({message: message});
	})
}


//retornem l'imatge de la publicació
function getImageFile(req,res){


	var image_file = req.params.imageFile;
	var path_file = './uploads/publications/' + image_file;

	//comprovem si existeixen les imatges de la publicació
	fs.exists(path_file, (exists) =>{

		if(exists){
			res.sendFile(path.resolve(path_file)); //rebem el directori i l'arxiu
		}else{
			//sino ens retorna un missatge d'error
			res.status(200).send({message: "No s'ha trobat cap coincidència"});
		}
	})
	

}


//exportem els moduls pertinents
module.exports = {
	savePublication,
	getPublications,
	getPublication,
	deletePublication,
	uploadImage,
	getImageFile,
	getPublicationsUser

}
