'use strict' //Ens permet emprar les derreres característiques de Javascript 
var bcrypt = require('bcrypt-nodejs'); // requerim bcrypt per al xifrat de contrasseñes
var User = require('../models/user');// carreguem el model d'usuari
var Follow = require('../models/follow'); // carreguem el model de seguiment
var Publication = require('../models/publication'); // carreguem el model de publicacions

var jwt = require('../services/jwt'); // carrega de token
var mongoosePaginate = require('mongoose-pagination'); // carreguem mongoose paginate
var fs = require('fs');// importam file system
var path = require('path');// importem la llibreria per a treballar amb la ruta dels fitxers

// funció per a donar d'alta un usuari
function saveUser(req,res){
	var params = req.body; // recollim els parametres arribats per POST i els guardem
	var user = new User();
	// Si els parametres obtinguts son correctes
	if(params.name && params.surname && params.nick && params.email && params.password){
		// doncs afegim les dades a l'objecte d'usuari
		user.name = params.name;
		user.surname = params.surname;
		user.nick = params.nick;
		user.email = params.email;
		user.role = params.role;
		user.image = 'null';

		// apliquem un control per a no permetre usuaris duplicats
		User.find({ $or: [

			{nick: user.nick.toLowerCase()}, // cerquem si aquest nick ja existeix
			{email: user.email.toLowerCase()} // cerquem si aquest mail ja existeix

			]}).exec((err,users) =>{ // apliquem una funció de callback
				if(err) return res.status(500).send({message: 'Hi ha agut un error, torna a intentar-ho!!'});//si hi ha error, doncs..
				if(users && users.length >= 1){
					 return res.status(500).send({message: 'Usuari no disponible'});
				}else{
					// guardem la contrassenya encriptada
					bcrypt.hash(params.password,null,null, (err,hash) =>{
						user.password = hash; 
						user.save((err,userStored) =>{
							if(err) return res.status(500).send({message: 'Hi ha agut un error, torna a intentar-ho'});//si hi ha error, doncs..
							if(userStored){
									return res.status(200).send({user: userStored}); // retornem l'objecte de l'usuari desat
							}else{
									return res.status(404).send({message: 'Hi ha hagut un error, torna a intentar-ho'})
							}
						});// guardem l'usuari
					});// xifram la contrassenya
				}
			});	

	}else{
		return res.status(200).send({  // si hi ha alguna mancança al formulari, doncs...
			message: 'Hi ha camps no omplerts correctament!!'
		})
	}
}


// funció per a dur a terme el login d'usuari
function loginUser(req,res){
	var params = req.body;
	var email = params.email;
	var password = params.password;
	User.findOne({email: email}, (err, user) =>{
			if(err) return res.status(500).send({message: 'Error del sistema'})
			if(user){
				bcrypt.compare(password, user.password, (err, check) =>{ // cerquem si aquest usuari i contrassenya concorden
					if(check){//si el check es correcte, retornem les dades d'usuari

							if(params.gettoken){// si es rep un token, doncs..
								// generem i retornem el token
								return res.status(200).send({
										token: jwt.createToken(user)
								});

							}else{ // si son dades planes, doncs..
								user.password = undefined;// d'aquesta manera evitem que es retorni e valor de la pwd a l'objecte json
								res.status(200).send({user});// retornem les dades d'usuari
							}

					}else{
						//si hi ha error, retornen un 404
						res.status(404).send({message:'Usuari o contrassenya incorrectes'});
					}

				})
	
			}else{
				// si hi ha error, retornem un 404
				res.status(404).send({message:'Usuari o contrassenya incorrectes!'});
			}

	})
}


//obtenir les dades d'un usuari
function getUser(req, res) {
    var userId = req.params.id;

    User.findById(userId, (err, user) => {
        if (!user) return res.status(404).send({message: "User Not Found."});
        if (err) return res.status(500).send({message: "Request Error."});

        followThisUser(req.user.sub, userId).then((value) => {
            return res.status(200).send({
                user,
                following: value.following,
                followed: value.followed
            });
        });
    });
}

//obtenim nom de l'usuari connectat
function getUsername(req, res) {
    var userId = req.params.id;

    User.findById(userId, (err, user) => {
        if (!user) return res.status(404).send({message: "User Not Found."});
        if (err) return res.status(500).send({message: "Request Error."});

        followThisUser(req.user.sub, userId).then((value) => {
            return res.status(200).send({
                user
            });
        });
    });
}



async function followThisUser(identity_user_id, user_id) {
    var following = await Follow.findOne({ user: identity_user_id, followed: user_id }).exec()
        .then((following) => {
            return following;
        })
        .catch((err) => {
            return handleError(err);
        });
    var followed = await Follow.findOne({ user: user_id, followed: identity_user_id }).exec()
        .then((followed) => {
            return followed;
        })
        .catch((err) => {
            return handleError(err);
        });

    return {
        following: following,
        followed: followed
    };
}
// actualitzar les dades d'usuari
function updateUser(req,res){
	var userId = req.params.id;
	var update = req.body;
	delete update.password; // esborrem la propiedad de password
	if(userId != req.user.sub){ // comprovem si l'usuari a modificar es l'usuari conectat
		return res.status(500).send({message: 'No disposes de permís per a modificar aquest usuari!!'});
	}
		User.find({ $or: [

			{nick: update.nick.toLowerCase()}, // cerquem si aquest nick ja existeix
			{email: update.email.toLowerCase()} // cerquem si aquest mail ja existeix

		]}).exec((err,users) =>{ // apliquem una funció de callback
				var user_isset = false;
				users.forEach((user) =>{
					if(user._id != userId) user_isset = true;
				});

				if(user_isset) return res.status(404).send({message: 'Aquest nom ja es utilitzat'});
			
				User.findByIdAndUpdate(userId, update, {new: true}, (err, userUpdated) =>{ 
				// si la petició no retorna l'objecte actualizat, doncs..
				if(err) return res.status(500).send({message: 'Hi ha agut un error, torna a intentar-ho!!'});
				// Si no s'ha rebut l'actualització de l'usuari..
				if(!userUpdated) return res.status(404).send({message: 'Hi ha agut un error, torna a intentar-ho!!'});
				// si tot ha anat be, doncs retorna l'objecte actualitzat
				return res.status(200).send({user: userUpdated});
			});
	
			});
	}


//retornan llistat d'usuaris paginats
function getUsers(req,res){
	var identity_user_id = req.user.sub; // obtenim id de l'usuari connectat
	var page = 1; // cream variable page
	if(req.params.page){
		page = req.params.page; // obtenim la página de l'usuari connectat

	}

	var itemsPerPage = 5; // definim la quantitat d'elements a mostrar a cada página
	User.find().sort('_id').paginate(page, itemsPerPage, (err,users,total) =>{  // obtenim els usuaris de la base de dades ordenats per ordre de creació
		if(err) return res.status(500).send({message: 'Hi ha agut un error, torna a intentar-ho!!'}); //si hi ha error feim un return
		if(!users) return res.status(404).send({message: 'No hi ha usuaris disponibles'});
		followingUsersId(identity_user_id).then((value)=>{
			return res.status(200).send({
			total_users:total,
			pages: Math.ceil(total/itemsPerPage), // feim el redondeig per a treure les pagines
			users_following: value.following,
			users_follow_me: value.followed,
			items_per_page: itemsPerPage,
			users
			
		});

		})	
		
	} )

}

// cream una funció asícrona per a obtenir un array de usuaris seguits i usuaris que ens segueixen (usuari connectat)

async function followingUsersId(user_id){
		// guardem un array d'usuaris els quals estic seguint
		var following = await Follow.find({"user":user_id}).select({'_id':0,'_v':0,'user':0});
		var followed = await Follow.find({"followed":user_id}).select({'_id':0,'_v':0,'followed':0});

		var followings_clean = []; //obtenim el llistat sense els camps que no necessitem
		
			following.forEach((follow) =>{
				followings_clean.push(follow.followed);
				
			});

		var followeds_clean = []; //obtenim el llistat sense els camps que no necessitem
			followed.forEach((follow) =>{
				followeds_clean.push(follow.user);
			});


		return{
			following: followings_clean,
			followed: followeds_clean,
			
		}


}

// funcio per a gestionar un contador d'usuaris seguits i da publicacions
function getCounters(req,res){
		//si rebrem parámetres per la url, feim una cridada a getCountFollows
		 var userId = req.user.sub;
		 if(req.params.id){
		 	userId = req.params.id;
		 }
		 getCountFollows(userId).then((value) =>{
		 		return res.status(200).send(value);

		 	})

}

//funció asíncrona per a la gestió de contadors de seguiment
async function getCountFollows(user_id){
	try{

		//var following = await Follow.countDocuments({"user": user_id});
		var following = await Follow.countDocuments({"user":user_id});

	    var followed = await Follow.countDocuments({"followed":user_id});

	    var publications = await Publication.countDocuments({"user":user_id});
	return{
		following: following,
		followed: followed,
		publications: publications

	}

   }catch(error){
   		return res.status(404).send({message: 'Hi ha hagut un error'});
   }
	
}


// cream funció per a la pujada d'imàtges de perfil d'usuari
function uploadImage(req,res){

	var userId = req.params.id;
	if(userId != req.user.sub){ // comprovem si l'usuari a modificar es l'usuari conectat
		 return notUploadUserImage(res, file_path, {message:'No disposes de permís per a modificar aquest usuari!!'});
	}
	if(req.files){

		var file_path = req.files.image.path; // obtenim la ruta de l'arxiu
		var file_split =file_path.split('\\');// separem el path en directoris i subdirectoris
		var file_name = file_split[2]; //obtenim el nom de l'arxiu
		var ext_split = file_name.split('\.');//separem per a obtenir l'extensio
		var file_ext = ext_split[1];//obtenim el tipus d'extensio
		

		//gestionem el control d'extensions permeses
		if(file_ext == 'png' || file_ext == 'jpg' || file_ext == 'jpeg' || file_ext == 'gif'){
			//actualitzam la imàtge d'usuari
			User.findByIdAndUpdate(userId, {image: file_name}, {new:true} , (err,userUpdated) =>{
				// si la petició no retorna l'objecte actualizat, doncs..
				if(err) return res.status(500).send({message: 'Hi ha agut un error, torna a intentar-ho!!'});
				// Si no s'ha rebut l'actualització de l'usuari..
				if(!userUpdated) return res.status(404).send({message: 'Hi ha agut un error, torna a intentar-ho!!'});
				// si tot ha anat be, doncs retorna l'objecte actualitzat
				return res.status(200).send({user: userUpdated});
			})

		}else{
			// no permetem extensions no indicades al condicional
			 return notUploadUserImage(res,file_path, 'Extensió no permesa!!');

		}

	}else{
		return res.status(200).send({message: 'Hi ha hagut un problema!!'});
	}

}

// creem una funció per a no permetre la pujada d'arxius

function notUploadUserImage(res,file_path,message){
	fs.unlink(file_path, (err)=>{
		return res.status(200).send({message: message});
	})
}


//retornem l'imatge del perfil d'un usuari
function getImageFile(req,res){


	var image_file = req.params.imageFile;
	var path_file = './uploads/users/' + image_file;

	//comprovem si existeix la imàtge d'usuari
	fs.exists(path_file, (exists) =>{

		if(exists){
			res.sendFile(path.resolve(path_file)); //rebem el directori i l'arxiu
		}else{
			//sino ens retorna un missatge d'error
			res.status(200).send({message: "No s'ha trobat cap coincidència"});
		}
	})
	

}



//funció de proves
function home(req,res){
	res.status(200).send({
		message: 'SocialCMR by amovan@uoc.edu'
	});
};

function prova(req,res){
	res.status(200).send({
		message: 'Servidor SocialCMR en proves.'
	});
};
module.exports = { // exportem els métodes
	home,
	prova,
	saveUser,
	loginUser,
	getUser,
	getUsername,
	getUsers,
	updateUser,
	uploadImage,
	getImageFile,
	getCounters
}