//definim els serveis i els injectem a una altre classe
import {Injectable} from '@angular/core';
//importem httpClient per a poder fer peticions ajax i HttpHeaders per a poder enviar capçaleres
import {HttpClient, HttpHeaders} from '@angular/common/http';
//importem l'observable per a poder recollir les respostes de l'api
import {Observable} from 'rxjs/observable';
//importem el model d'usuari
import {User} from '../models/user';
//importam la connexió a l'api
import {Global} from './global';


//definim el decorador injectable per a poder ser utilitzat desde qualsevol classe del client
@Injectable()
export class UserService{
  public url:string;
  public identity;
  public token;
  public stats;
  //creem el constructor per a la connexió
  constructor( public _http: HttpClient ){
    this.url = Global.url;
  }
  register(user: User): Observable<any>{
    let params = JSON.stringify(user);
    let headers = new HttpHeaders().set('Content-Type','application/json');
    return this._http.post(this.url + 'register', params, {headers:headers});
  }


  signup(user, gettoken = null): Observable<any>{

    if(gettoken != null){
      user.gettoken = gettoken;
    }
    let params = JSON.stringify(user);
    let headers = new HttpHeaders().set('Content-Type','application/json');
    return this._http.post(this.url + 'login', params, {headers:headers});
  }

  //consultam les dades de localStorage per poder extreure les dades d'usuari
  getIdentity(){
    //convertim la variable identity a un objecte de javascript
    let identity = JSON.parse(localStorage.getItem('identity'));
    if(identity != "undefined"){
      this.identity = identity;
    }else{
      this.identity = null;
    }
    return this.identity;

  }

  gettoken(){
    //convertim la variable token a un objecte de javascript
    let token = localStorage.getItem('token');
    if(token != "undefined"){
       this.token = token;
    }else{
      this.token = null;
    }
      return this.token;
    }

    getStats(){

        let stats = JSON.parse(localStorage.getItem('stats'));
        if(stats != "undefined"){
          this.stats = stats;
        }else{
          this.stats = null;
        }
        return this.stats;
    }

    getCounters(userId = null): Observable<any>{
      let headers = new HttpHeaders().set('Content-Type', 'application/json').set('Authorization', this.gettoken());

      //comprovam si hi ha usuari autenticat
      if(userId != null){
              return this._http.get(this.url + 'counters/' + userId, {headers:headers});
      }else{
              return this._http.get(this.url + 'counters/',{headers:headers});
      }
    }


    updateUser(user: User): Observable<any>{
      let params = JSON.stringify(user);//convertim els parametres a objecte
      let headers = new HttpHeaders().set('Content-Type', 'application/json').set('Authorization', this.gettoken());//decodificam el token
      return this._http.put(this.url + 'update-user/' + user._id, params,{headers:headers});

    }

    getUsers(page = null): Observable<any>{

        let headers = new HttpHeaders().set('Content-Type', 'application/json').set('Authorization', this.gettoken());//decodificam el token
        return this._http.get(this.url + 'users/'+ page,{headers:headers});

    }

    getUser(id): Observable<any>{
        let headers = new HttpHeaders().set('Content-Type', 'application/json').set('Authorization', this.gettoken());//decodificam el token
        return this._http.get(this.url + 'user/'+ id,{headers:headers});

    }


}
